@@ -65,6 +65,7 @@ typedef struct
   int title_lines;
   int subtitle_lines;
   GtkWidget *activatable_widget;
+  GBinding *activatable_binding;
 } AdwActionRowPrivate;
 
 static void adw_action_row_buildable_init (GtkBuildableIface *iface);
@@ -315,7 +316,6 @@ adw_action_row_class_init (AdwActionRowClass *klass)
 
   /**
    * AdwActionRow::activated:
-   * @self: the `AdwActionRow` instance
    *
    * This signal is emitted after the row has been activated.
    *
@@ -396,7 +396,7 @@ adw_action_row_new (void)
 
 /**
  * adw_action_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Gets the subtitle for @self.
  *
@@ -418,7 +418,7 @@ adw_action_row_get_subtitle (AdwActionRow *self)
 
 /**
  * adw_action_row_set_subtitle: (attributes org.gtk.Method.set_property=subtitle)
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @subtitle: the subtitle
  *
  * Sets the subtitle for @self.
@@ -445,7 +445,7 @@ adw_action_row_set_subtitle (AdwActionRow *self,
 
 /**
  * adw_action_row_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Gets the icon name for @self.
  *
@@ -467,7 +467,7 @@ adw_action_row_get_icon_name (AdwActionRow *self)
 
 /**
  * adw_action_row_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @icon_name: (nullable): the icon name
  *
  * Sets the icon name for @self.
@@ -496,7 +496,7 @@ adw_action_row_set_icon_name (AdwActionRow *self,
 
 /**
  * adw_action_row_get_activatable_widget: (attributes org.gtk.Method.get_property=activatable-widget)
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Gets the widget activated when @self is activated.
  *
@@ -524,13 +524,14 @@ activatable_widget_weak_notify (gpointer  data,
   AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
 
   priv->activatable_widget = NULL;
+  priv->activatable_binding = NULL;
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACTIVATABLE_WIDGET]);
 }
 
 /**
  * adw_action_row_set_activatable_widget: (attributes org.gtk.Method.set_property=activatable-widget)
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @widget: (nullable): the target widget
  *
  * Sets the widget to activate when @self is activated.
@@ -551,6 +552,8 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
   if (priv->activatable_widget == widget)
     return;
 
+  g_clear_pointer (&priv->activatable_binding, g_binding_unbind);
+
   if (priv->activatable_widget)
     g_object_weak_unref (G_OBJECT (priv->activatable_widget),
                          activatable_widget_weak_notify,
@@ -562,7 +565,11 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
     g_object_weak_ref (G_OBJECT (priv->activatable_widget),
                        activatable_widget_weak_notify,
                        self);
-    gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), TRUE);
+
+    priv->activatable_binding =
+      g_object_bind_property (widget, "sensitive",
+                              self, "activatable",
+                              G_BINDING_SYNC_CREATE);
   }
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACTIVATABLE_WIDGET]);
@@ -570,7 +577,7 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
 
 /**
  * adw_action_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Gets the number of lines at the end of which the title label will be
  * ellipsized.
@@ -596,7 +603,7 @@ adw_action_row_get_title_lines (AdwActionRow *self)
 
 /**
  * adw_action_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
  * Sets the number of lines at the end of which the title label will be
@@ -630,7 +637,7 @@ adw_action_row_set_title_lines (AdwActionRow *self,
 
 /**
  * adw_action_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Gets the number of lines at the end of which the subtitle label will be
  * ellipsized.
@@ -656,7 +663,7 @@ adw_action_row_get_subtitle_lines (AdwActionRow *self)
 
 /**
  * adw_action_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
  * Sets the number of lines at the end of which the subtitle label will be
@@ -690,7 +697,7 @@ adw_action_row_set_subtitle_lines (AdwActionRow *self,
 
 /**
  * adw_action_row_add_prefix:
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @widget: a widget
  *
  * Adds a prefix widget to @self.
@@ -714,7 +721,7 @@ adw_action_row_add_prefix (AdwActionRow *self,
 
 /**
  * adw_action_row_add_suffix:
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @widget: a widget
  *
  * Adds a suffix widget to @self.
@@ -738,7 +745,7 @@ adw_action_row_add_suffix (AdwActionRow *self,
 
 /**
  * adw_action_row_remove:
- * @self: a `AdwActionRow`
+ * @self: an action row
  * @widget: the child to be removed
  *
  * Removes a child from @self.
@@ -769,7 +776,7 @@ adw_action_row_remove (AdwActionRow *self,
 
 /**
  * adw_action_row_activate:
- * @self: a `AdwActionRow`
+ * @self: an action row
  *
  * Activates @self.
  *
