@@ -83,11 +83,13 @@ static char *
 extract_initials_from_text (const char *text)
 {
   GString *initials;
-  g_autofree char *p = g_utf8_strup (text, -1);
-  g_autofree char *normalized = g_utf8_normalize (g_strstrip (p), -1, G_NORMALIZE_DEFAULT_COMPOSE);
+  char *p = g_utf8_strup (text, -1);
+  char *normalized = g_utf8_normalize (g_strstrip (p), -1, G_NORMALIZE_DEFAULT_COMPOSE);
   gunichar unichar;
   char *q = NULL;
 
+  g_clear_pointer (&p, g_free);
+
   if (normalized == NULL)
     return NULL;
 
@@ -104,6 +106,8 @@ extract_initials_from_text (const char *text)
       g_string_append_unichar (initials, unichar);
   }
 
+  g_free (normalized);
+
   return g_string_free (initials, FALSE);
 }
 
@@ -121,29 +125,33 @@ update_visibility (AdwAvatar *self)
 static void
 set_class_color (AdwAvatar *self)
 {
-  g_autofree GRand *rand = NULL;
-  g_autofree char *new_class = NULL;
-  g_autofree char *old_class = g_strdup_printf ("color%d", self->color_class);
+  char *old_class, *new_class;
 
+  old_class = g_strdup_printf ("color%d", self->color_class);
   gtk_widget_remove_css_class (self->gizmo, old_class);
 
   if (self->text == NULL || strlen (self->text) == 0) {
     /* Use a random color if we don't have a text */
-    rand = g_rand_new ();
+    GRand *rand = g_rand_new ();
+
     self->color_class = g_rand_int_range (rand, 1, NUMBER_OF_COLORS);
+
+    g_rand_free (rand);
   } else {
     self->color_class = (g_str_hash (self->text) % NUMBER_OF_COLORS) + 1;
   }
 
   new_class = g_strdup_printf ("color%d", self->color_class);
-
   gtk_widget_add_css_class (self->gizmo, new_class);
+
+  g_free (old_class);
+  g_free (new_class);
 }
 
 static void
 update_initials (AdwAvatar *self)
 {
-  g_autofree char *initials = NULL;
+  char *initials;
 
   if (gtk_image_get_paintable (self->custom_image) != NULL ||
       !self->show_initials ||
@@ -154,6 +162,8 @@ update_initials (AdwAvatar *self)
   initials = extract_initials_from_text (self->text);
 
   gtk_label_set_label (self->label, initials);
+
+  g_free (initials);
 }
 
 static void
@@ -203,6 +213,24 @@ update_font_size (AdwAvatar *self)
   pango_attr_list_unref (attributes);
 }
 
+static void
+update_custom_image_snapshot (AdwAvatar *self)
+{
+  GtkSnapshot *snapshot = gtk_snapshot_new ();
+  GdkPaintable *square_image;
+  int height = gdk_paintable_get_intrinsic_height (self->custom_image_source);
+  int width = gdk_paintable_get_intrinsic_width (self->custom_image_source);
+  int size = MIN (width, height);
+
+  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - width) / 2.f, (size - height) / 2.f));
+  gdk_paintable_snapshot (self->custom_image_source, snapshot, width, height);
+
+  square_image = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
+  gtk_image_set_from_paintable (self->custom_image, square_image);
+
+  g_object_unref (square_image);
+}
+
 static void
 adw_avatar_get_property (GObject    *object,
                          guint       property_id,
@@ -448,7 +476,7 @@ adw_avatar_new (int         size,
 
 /**
  * adw_avatar_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  *
  * Gets the name of an icon to use as a fallback.
  *
@@ -466,7 +494,7 @@ adw_avatar_get_icon_name (AdwAvatar *self)
 
 /**
  * adw_avatar_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @icon_name: (nullable): the icon name
  *
  * Sets the name of an icon to use as a fallback.
@@ -494,7 +522,7 @@ adw_avatar_set_icon_name (AdwAvatar  *self,
 
 /**
  * adw_avatar_get_text: (attributes org.gtk.Method.get_property=text)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  *
  * Gets the text used to generate the fallback initials and color.
  *
@@ -513,7 +541,7 @@ adw_avatar_get_text (AdwAvatar *self)
 
 /**
  * adw_avatar_set_text: (attributes org.gtk.Method.set_property=text)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @text: (nullable): the text used to get the initials and color
  *
  * Sets the text used to generate the fallback initials and color.
@@ -543,7 +571,7 @@ adw_avatar_set_text (AdwAvatar  *self,
 
 /**
  * adw_avatar_get_show_initials: (attributes org.gtk.Method.get_property=show-initials)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  *
  * Gets whether initials are used instead of an icon on the fallback avatar.
  *
@@ -561,7 +589,7 @@ adw_avatar_get_show_initials (AdwAvatar *self)
 
 /**
  * adw_avatar_set_show_initials: (attributes org.gtk.Method.set_property=show-initials)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @show_initials: whether to use initials instead of an icon as fallback
  *
  * Sets whether to use initials instead of an icon on the fallback avatar.
@@ -588,7 +616,7 @@ adw_avatar_set_show_initials (AdwAvatar *self,
 
 /**
  * adw_avatar_get_custom_image: (attributes org.gtk.Method.get_property=custom-image)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  *
  * Gets the custom image paintable.
  *
@@ -606,7 +634,7 @@ adw_avatar_get_custom_image (AdwAvatar *self)
 
 /**
  * adw_avatar_set_custom_image: (attributes org.gtk.Method.set_property=custom-image)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @custom_image: (nullable) (transfer none): a custom image
  *
  * Sets the custom image paintable.
@@ -623,6 +651,11 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
   if (self->custom_image_source == custom_image)
     return;
 
+  if (self->custom_image_source) {
+    g_signal_handlers_disconnect_by_func (self->custom_image_source,
+                                          update_custom_image_snapshot, self);
+  }
+
   g_set_object (&self->custom_image_source, custom_image);
 
   if (custom_image) {
@@ -632,15 +665,10 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
     if (height == width) {
       gtk_image_set_from_paintable (self->custom_image, custom_image);
     } else {
-      GtkSnapshot *snapshot = gtk_snapshot_new ();
-      g_autoptr (GdkPaintable) square_image = NULL;
-      int size = MIN (width, height);
-
-      gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - width) / 2.f, (size - height) / 2.f));
-      gdk_paintable_snapshot (custom_image, snapshot, width, height);
-
-      square_image = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
-      gtk_image_set_from_paintable (self->custom_image, square_image);
+      update_custom_image_snapshot (self);
+      g_signal_connect_swapped (custom_image, "invalidate-contents",
+                                G_CALLBACK (update_custom_image_snapshot),
+                                self);
     }
 
     gtk_widget_add_css_class (self->gizmo, "image");
@@ -657,7 +685,7 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
 
 /**
  * adw_avatar_get_size: (attributes org.gtk.Method.get_property=size)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  *
  * Gets the size of the avatar.
  *
@@ -675,7 +703,7 @@ adw_avatar_get_size (AdwAvatar *self)
 
 /**
  * adw_avatar_set_size: (attributes org.gtk.Method.set_property=size)
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @size: The size of the avatar
  *
  * Sets the size of the avatar.
@@ -710,7 +738,7 @@ adw_avatar_set_size (AdwAvatar *self,
 
 /**
  * adw_avatar_draw_to_texture:
- * @self: a `AdwAvatar`
+ * @self: an avatar
  * @scale_factor: The scale factor
  *
  * Renders @self into a [class@Gdk.Texture] at @scale_factor.
@@ -725,7 +753,8 @@ GdkTexture *
 adw_avatar_draw_to_texture (AdwAvatar *self,
                             int        scale_factor)
 {
-  g_autoptr (GskRenderNode) node = NULL;
+  GdkTexture *result;
+  GskRenderNode *node;
   GtkSnapshot *snapshot;
   GtkNative *native;
   GskRenderer *renderer;
@@ -751,5 +780,9 @@ adw_avatar_draw_to_texture (AdwAvatar *self,
   native = gtk_widget_get_native (GTK_WIDGET (self));
   renderer = gtk_native_get_renderer (native);
 
-  return gsk_renderer_render_texture (renderer, node, &GRAPHENE_RECT_INIT (0, 0, size, size));
+  result = gsk_renderer_render_texture (renderer, node, &GRAPHENE_RECT_INIT (0, 0, size, size));
+
+  gsk_render_node_unref (node);
+
+  return result;
 }
