@@ -89,7 +89,7 @@ static GParamSpec *props[LAST_PROP];
 static char *
 strip_mnemonic (const char *src)
 {
-  g_autofree char *new_str = g_new (char, strlen (src) + 1);
+  char *new_str = g_new (char, strlen (src) + 1);
   char *dest = new_str;
   gboolean underscore = FALSE;
 
@@ -101,6 +101,8 @@ strip_mnemonic (const char *src)
     if (c == (gunichar) -1) {
       g_warning ("Invalid input string");
 
+      g_free (new_str);
+
       return NULL;
     }
 
@@ -124,7 +126,7 @@ strip_mnemonic (const char *src)
 
   *dest = 0;
 
-  return g_steal_pointer (&new_str);
+  return new_str;
 }
 
 static gboolean
@@ -132,8 +134,8 @@ filter_search_results (AdwPreferencesRow    *row,
                        AdwPreferencesWindow *self)
 {
   AdwPreferencesWindowPrivate *priv = adw_preferences_window_get_instance_private (self);
-  g_autofree char *terms = NULL;
-  g_autofree char *title = NULL;
+  char *terms, *title;
+  gboolean result = FALSE;
 
   g_assert (ADW_IS_PREFERENCES_ROW (row));
 
@@ -145,21 +147,26 @@ filter_search_results (AdwPreferencesRow    *row,
 
     if (stripped_title) {
       g_free (title);
+
       title = stripped_title;
     }
   }
 
-  if (!!strstr (title, terms))
-    return TRUE;
-
-  if (ADW_IS_ACTION_ROW (row)) {
-    g_autofree char *subtitle = g_utf8_casefold (adw_action_row_get_subtitle (ADW_ACTION_ROW (row)), -1);
+  if (!!strstr (title, terms)) {
+    result = TRUE;
+  } else if (ADW_IS_ACTION_ROW (row)) {
+    char *subtitle = g_utf8_casefold (adw_action_row_get_subtitle (ADW_ACTION_ROW (row)), -1);
 
     if (!!strstr (subtitle, terms))
-      return TRUE;
+      result = TRUE;
+
+    g_free (subtitle);
   }
 
-  return FALSE;
+  g_free (title);
+  g_free (terms);
+
+  return result;
 }
 
 static int
@@ -187,7 +194,7 @@ create_search_row_subtitle (AdwPreferencesWindow *self,
 {
   GtkWidget *group, *page;
   const char *group_title = NULL;
-  g_autofree char *page_title = NULL;
+  char *page_title = NULL;
 
   group = gtk_widget_get_ancestor (row, ADW_TYPE_PREFERENCES_GROUP);
 
@@ -209,20 +216,23 @@ create_search_row_subtitle (AdwPreferencesWindow *self,
       page_title = g_strdup (title);
 
     if (g_strcmp0 (page_title, "") == 0)
-      page_title = NULL;
+      g_clear_pointer (&page_title, g_free);
   }
 
   if (group_title) {
-    if (get_n_pages (self) > 1)
-      return g_strdup_printf ("%s → %s", page_title ? page_title : _("Untitled page"), group_title);
+    gchar *result;
 
-    return g_strdup (group_title);
+    if (get_n_pages (self) > 1)
+      result = g_strdup_printf ("%s → %s", page_title ? page_title : _("Untitled page"), group_title);
+    else
+      result = g_strdup (group_title);
+
+    g_free (page_title);
+
+    return result;
   }
 
-  if (page_title)
-    return g_steal_pointer (&page_title);
-
-  return NULL;
+  return page_title;
 }
 
 static GtkWidget *
@@ -231,7 +241,7 @@ new_search_row_for_preference (AdwPreferencesRow    *row,
 {
   AdwActionRow *widget;
   GtkWidget *page;
-  g_autofree char *subtitle = NULL;
+  char *subtitle;
 
   g_assert (ADW_IS_PREFERENCES_ROW (row));
 
@@ -247,6 +257,8 @@ new_search_row_for_preference (AdwPreferencesRow    *row,
   g_object_set_data (G_OBJECT (widget), "page", page);
   g_object_set_data (G_OBJECT (widget), "row", row);
 
+  g_clear_pointer (&subtitle, g_free);
+
   return GTK_WIDGET (widget);
 }
 
@@ -576,7 +588,8 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
    * - Horizontal scrolling on touchpads (usually two-finger swipe)
    * - Back mouse button
    *
-   * The keyboard back key is also supported, as well as the Alt+← shortcut.
+   * The keyboard back key is also supported, as well as the
+   * <kbd>Alt</kbd>+<kbd>←</kbd> shortcut.
    *
    * For right-to-left locales, gestures and shortcuts are reversed.
    *
@@ -623,11 +636,12 @@ adw_preferences_window_class_init (AdwPreferencesWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, stop_search_cb);
 }
 
-static gpointer
-preferences_page_to_rows (gpointer page,
-                          gpointer user_data)
+static GListModel *
+preferences_page_to_rows (AdwViewStackPage *page)
 {
-  GtkWidget *child = adw_view_stack_page_get_child (ADW_VIEW_STACK_PAGE (page));
+  GtkWidget *child = adw_view_stack_page_get_child (page);
+
+  g_object_unref (page);
 
   return adw_preferences_page_get_rows (ADW_PREFERENCES_PAGE (child));
 }
@@ -648,7 +662,10 @@ adw_preferences_window_init (AdwPreferencesWindow *self)
 
   model = G_LIST_MODEL (adw_view_stack_get_pages (priv->pages_stack));
   model = G_LIST_MODEL (gtk_filter_list_model_new (model, GTK_FILTER (gtk_bool_filter_new (expr))));
-  model = G_LIST_MODEL (gtk_map_list_model_new (model, preferences_page_to_rows, NULL, NULL));
+  model = G_LIST_MODEL (gtk_map_list_model_new (model,
+                                                (GtkMapListModelMapFunc) preferences_page_to_rows,
+                                                NULL,
+                                                NULL));
   model = G_LIST_MODEL (gtk_flatten_list_model_new (model));
   priv->filter_model = gtk_filter_list_model_new (model, priv->filter);
 
@@ -696,7 +713,7 @@ adw_preferences_window_new (void)
 
 /**
  * adw_preferences_window_get_search_enabled: (attributes org.gtk.Method.get_property=search-enabled)
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  *
  * Gets whether search is enabled for @self.
  *
@@ -718,7 +735,7 @@ adw_preferences_window_get_search_enabled (AdwPreferencesWindow *self)
 
 /**
  * adw_preferences_window_set_search_enabled: (attributes org.gtk.Method.set_property=search-enabled)
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @search_enabled: whether search is enabled
  *
  * Sets whether search is enabled for @self.
@@ -754,7 +771,7 @@ adw_preferences_window_set_search_enabled (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_set_can_navigate_back: (attributes org.gtk.Method.set_property=can-navigate-back)
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @can_navigate_back: the new value
  *
  * Sets whether gestures and shortcuts for closing subpages are enabled.
@@ -783,7 +800,7 @@ adw_preferences_window_set_can_navigate_back (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_get_can_navigate_back: (attributes org.gtk.Method.get_property=can-navigate-back)
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  *
  * Gets whether gestures and shortcuts for closing subpages are enabled.
  *
@@ -805,7 +822,7 @@ adw_preferences_window_get_can_navigate_back (AdwPreferencesWindow *self)
 
 /**
  * adw_preferences_window_present_subpage:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @subpage: the subpage
  *
  * Sets @subpage as the window's subpage and opens it.
@@ -842,14 +859,12 @@ adw_preferences_window_present_subpage (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_close_subpage:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  *
  * Closes the current subpage.
  *
  * If there is no presented subpage, this does nothing.
  *
- * See [method@PreferencesWindow.close_subpage].
- *
  * Since: 1.0
  */
 void
@@ -869,7 +884,7 @@ adw_preferences_window_close_subpage (AdwPreferencesWindow *self)
 
 /**
  * adw_preferences_window_add:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @page: the page to add
  *
  * Adds a preferences page to @self.
@@ -898,7 +913,7 @@ adw_preferences_window_add (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_remove:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @page: the page to remove
  *
  * Removes a page from @self.
@@ -924,7 +939,7 @@ adw_preferences_window_remove (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_get_visible_page:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  *
  * Gets the currently visible page of @self.
  *
@@ -946,7 +961,7 @@ adw_preferences_window_get_visible_page (AdwPreferencesWindow *self)
 
 /**
  * adw_preferences_window_set_visible_page:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @page: a page of @self
  *
  * Makes @page the visible page of @self.
@@ -969,7 +984,7 @@ adw_preferences_window_set_visible_page (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_get_visible_page_name:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  *
  * Gets the name of currently visible page of @self.
  *
@@ -991,7 +1006,7 @@ adw_preferences_window_get_visible_page_name (AdwPreferencesWindow *self)
 
 /**
  * adw_preferences_window_set_visible_page_name:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @name: the name of the page to make visible
  *
  * Makes the page with the given name visible.
@@ -1013,7 +1028,7 @@ adw_preferences_window_set_visible_page_name (AdwPreferencesWindow *self,
 
 /**
  * adw_preferences_window_add_toast:
- * @self: a `AdwPreferencesWindow`
+ * @self: a preferences window
  * @toast: (transfer full): a toast
  *
  * Displays @toast.
