@@ -232,7 +232,7 @@ settings_portal_changed_cb (GDBusProxy  *proxy,
     return;
   }
 
-  if (!g_strcmp0 (namespace, "org.gnome.desktop.interface.a11y") &&
+  if (!g_strcmp0 (namespace, "org.gnome.desktop.a11y.interface") &&
       !g_strcmp0 (name, "high-contrast")) {
     set_high_contrast (self, g_variant_get_boolean (value));
 
@@ -240,6 +240,8 @@ settings_portal_changed_cb (GDBusProxy  *proxy,
 
     return;
   }
+
+  g_variant_unref (value);
 }
 
 static void
@@ -285,7 +287,7 @@ init_portal (AdwSettings *self)
     g_variant_unref (variant);
   }
 
-  if (read_portal_setting (self, "org.gnome.desktop.interface.a11y",
+  if (read_portal_setting (self, "org.gnome.desktop.a11y.interface",
                            "high-contrast", "b", &variant)) {
     self->has_high_contrast = TRUE;
     self->high_contrast = g_variant_get_boolean (variant);
