@@ -811,7 +811,6 @@ adw_squeezer_size_allocate (GtkWidget *widget,
   for (l = self->children; l; l = l->next) {
     GtkWidget *child = NULL;
     int child_min, child_nat;
-    int for_size = -1;
     int compare_size;
 
     page = l->data;
@@ -826,29 +825,20 @@ adw_squeezer_size_allocate (GtkWidget *widget,
     if (self->orientation == GTK_ORIENTATION_VERTICAL) {
       compare_size = height;
 
-      if (gtk_widget_get_request_mode (child) == GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH)
-        for_size = width;
-
-      gtk_widget_measure (child, self->orientation, for_size,
-                          &child_min, NULL, NULL, NULL);
-
-      if (child_min <= height)
-        break;
+      gtk_widget_measure (child, self->orientation, -1,
+                          &child_min, &child_nat, NULL, NULL);
     } else {
       compare_size = width;
 
-      if (gtk_widget_get_request_mode (child) == GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT)
-        for_size = height;
-
-      gtk_widget_measure (child, self->orientation, for_size,
+      gtk_widget_measure (child, self->orientation, -1,
                           &child_min, &child_nat, NULL, NULL);
-
-      if (child_min <= compare_size && self->switch_threshold_policy == ADW_FOLD_THRESHOLD_POLICY_MINIMUM)
-        break;
-
-      if (child_nat <= compare_size && self->switch_threshold_policy == ADW_FOLD_THRESHOLD_POLICY_NATURAL)
-        break;
     }
+
+    if (child_min <= compare_size && self->switch_threshold_policy == ADW_FOLD_THRESHOLD_POLICY_MINIMUM)
+      break;
+
+    if (child_nat <= compare_size && self->switch_threshold_policy == ADW_FOLD_THRESHOLD_POLICY_NATURAL)
+      break;
   }
 
   if (l == NULL && self->allow_none)
@@ -961,8 +951,12 @@ adw_squeezer_measure (GtkWidget      *widget,
      * appearant size and position of a child to changes suddenly when a larger
      * child gets enabled/disabled.
      */
-    gtk_widget_measure (child, orientation, for_size,
-                        &child_min, &child_nat, NULL, NULL);
+    if (self->orientation == orientation)
+      gtk_widget_measure (child, orientation, -1,
+                          &child_min, &child_nat, NULL, NULL);
+    else
+      gtk_widget_measure (child, orientation, for_size,
+                          &child_min, &child_nat, NULL, NULL);
 
     if (self->orientation == orientation) {
       if (self->allow_none)
