@@ -15,7 +15,6 @@
 #define TOUCHPAD_BASE_DISTANCE_H 400
 #define TOUCHPAD_BASE_DISTANCE_V 300
 #define EVENT_HISTORY_THRESHOLD_MS 150
-#define SCROLL_MULTIPLIER 10
 #define MIN_ANIMATION_DURATION 100
 #define MAX_ANIMATION_DURATION 400
 #define VELOCITY_THRESHOLD_TOUCH 0.3
@@ -29,6 +28,12 @@
 #define DRAG_THRESHOLD_DISTANCE 16
 #define EPSILON 0.005
 
+#if GTK_CHECK_VERSION (4, 7, 0)
+#define SCROLL_MULTIPLIER 1
+#else
+#define SCROLL_MULTIPLIER 10
+#endif
+
 #define SIGN(x) ((x) > 0.0 ? 1.0 : ((x) < 0.0 ? -1.0 : 0.0))
 
 /**
@@ -670,6 +675,9 @@ handle_scroll_event (AdwSwipeTracker *self,
   is_vertical = (self->orientation == GTK_ORIENTATION_VERTICAL);
   distance = is_vertical ? TOUCHPAD_BASE_DISTANCE_V : TOUCHPAD_BASE_DISTANCE_H;
 
+  if (!event || gdk_event_get_event_type (event) != GDK_SCROLL)
+    return GDK_EVENT_PROPAGATE;
+
   if (gdk_scroll_event_get_direction (event) != GDK_SCROLL_SMOOTH)
     return GDK_EVENT_PROPAGATE;
 
@@ -914,6 +922,16 @@ adw_swipe_tracker_dispose (GObject *object)
   G_OBJECT_CLASS (adw_swipe_tracker_parent_class)->dispose (object);
 }
 
+static void
+adw_swipe_tracker_finalize (GObject *object)
+{
+  AdwSwipeTracker *self = ADW_SWIPE_TRACKER (object);
+
+  g_array_free (self->event_history, TRUE);
+
+  G_OBJECT_CLASS (adw_swipe_tracker_parent_class)->finalize (object);
+}
+
 static void
 adw_swipe_tracker_get_property (GObject    *object,
                                 guint       prop_id,
@@ -997,6 +1015,7 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
 
   object_class->constructed = adw_swipe_tracker_constructed;
   object_class->dispose = adw_swipe_tracker_dispose;
+  object_class->finalize = adw_swipe_tracker_finalize;
   object_class->get_property = adw_swipe_tracker_get_property;
   object_class->set_property = adw_swipe_tracker_set_property;
 
